<?php

declare(strict_types=1);

namespace Drupal\views_entity_form_row\Plugin\views\row;

use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\row\RowPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @ingroup views_row_plugins
 *
 * @ViewsRow(
 *   id = "views_entity_form_row",
 *   deriver = "Drupal\views_entity_form_row\Plugin\Derivative\EntityFormRow"
 * )
 */
final class EntityFormRow extends RowPluginBase {

  protected EntityDisplayRepositoryInterface $entityDisplayRepository;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityDisplayRepositoryInterface $entity_display_repository) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityDisplayRepository = $entity_display_repository;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_display.repository')
    );
  }

  /**
   * Does the row plugin support to add fields to its output.
   *
   */
  protected $usesFields = FALSE;

  /**
   * {@inheritdoc}
   */
  protected function defineOptions(): array {
    $options = parent::defineOptions();
    $options['form_mode'] = ['default' => 'default'];

    return $options;
  }

  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    parent::buildOptionsForm($form, $form_state);

    $form['form_mode'] = [
      '#type' => 'select',
      '#options' => $this->entityDisplayRepository->getFormModeOptions($this->definition['entity_type']),
      '#title' => $this->t('Form mode'),
      '#default_value' => $this->options['form_mode'],
    ];
  }

}
